
/*	File			CaseStudy_NotServ_Publish_Client_CPPDlg.cpp
	Purpose			Notification Service Client  (Publisher)
	Author			Richard Anthony	(R.J.Anthony@gre.ac.uk)
	Date			September 2014
*/

#include "stdafx.h"
#include "CaseStudy_NotServ_Publish_Client_CPP.h"
#include "CaseStudy_NotServ_Publish_Client_CPPDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CCaseStudy_NotServ_Publish_Client_CPPDlg::CCaseStudy_NotServ_Publish_Client_CPPDlg(CWnd* pParent /*= NULL*/)
	: CDialog(CCaseStudy_NotServ_Publish_Client_CPPDlg::IDD, pParent)
{
	m_bConnected = false;
	m_iTemperatureValue_Previous = 50;
	m_iHumidityValue_Previous = 50;			
	
	bool bResult = CreateResolveSocket();
	if(false == bResult)
	{
		OnBnClickedCancel();
	}

	bResult = CreateApplicationSocket();
	if(false == bResult)
	{
		OnBnClickedCancel();
	}
	
	SetUpLocalAddressStruct(m_LocalAddr, From_DIRECTORY_SERVICE_PORT);
	int iResult = bind(m_ResolveSock, (const SOCKADDR FAR*)&m_LocalAddr, sizeof(m_LocalAddr));
	if(SOCKET_ERROR == iResult)
	{		
		m_pEventLogList->AddString("bind failed");		
		OnBnClickedCancel();	// Error binding, close dialog
	}
	SetUpSendAddressStructForBroadcast(m_ResolveAddr, To_DIRECTORY_SERVICE_PORT);
}

CCaseStudy_NotServ_Publish_Client_CPPDlg::~CCaseStudy_NotServ_Publish_Client_CPPDlg()
{	
	closesocket(m_ResolveSock);
	closesocket(m_ApplicationSock);
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	m_pEventLogList = (CListBox*) GetDlgItem(IDC_Event_Log_LIST);
	m_pTemperature_Slider = (CSliderCtrl*) GetDlgItem(Temperature_SLIDER);
	m_pHumiditySlider = (CSliderCtrl*) GetDlgItem(Humidity_SLIDER);

	m_pTemperature_Slider->SetRangeMin(0);
	m_pTemperature_Slider->SetRangeMax(100);
	m_pTemperature_Slider->SetPos(50);
	m_pTemperature_Slider->EnableWindow(false);
	m_pHumiditySlider->SetRangeMin(0);
	m_pHumiditySlider->SetRangeMax(100);
	m_pHumiditySlider->SetPos(50);
	m_pHumiditySlider->EnableWindow(false);
	Contact_DirectoryService();
}

BEGIN_MESSAGE_MAP(CCaseStudy_NotServ_Publish_Client_CPPDlg, CDialog)
	ON_WM_TIMER()
	ON_BN_CLICKED(ClearEventLog_BUTTON, &CCaseStudy_NotServ_Publish_Client_CPPDlg::OnBnClicked_ClearEventLog_Button)
	ON_NOTIFY(NM_CUSTOMDRAW, Temperature_SLIDER, &CCaseStudy_NotServ_Publish_Client_CPPDlg::OnNMCustomdrawSlider)
	ON_NOTIFY(NM_CUSTOMDRAW, Humidity_SLIDER, &CCaseStudy_NotServ_Publish_Client_CPPDlg::OnNMCustomdraw_Humidity_Slider)
END_MESSAGE_MAP()

void CCaseStudy_NotServ_Publish_Client_CPPDlg::Contact_DirectoryService()
{
	SendResolveMessage();
	InitiateTimer(Client_WaitForDirectoryServiceReply_Event);	// wait for reply
}

bool CCaseStudy_NotServ_Publish_Client_CPPDlg::CreateResolveSocket()
{   // UDP, non-blocking, broadcast
	m_ResolveSock = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
	if(INVALID_SOCKET == m_ResolveSock)
	{
		return false;	// Error creating socket
	}

	unsigned long lNonBlocking = 1;
	int iError = ioctlsocket(m_ResolveSock, FIONBIO, &lNonBlocking);
	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting non-blocking behaviour
	}

	// Turn on Broadcast mode
	char cOpt[2];
	cOpt[0] = 1; // true
	cOpt[1] = 0; // null terminate the option array
	iError = setsockopt(m_ResolveSock, SOL_SOCKET, SO_BROADCAST, cOpt, sizeof(cOpt)); 
  	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting broadcast option
	}
	return true;
}

bool CCaseStudy_NotServ_Publish_Client_CPPDlg::CreateApplicationSocket()
{   // TCP, non-blocking
	m_ApplicationSock = socket(AF_INET, SOCK_STREAM, PF_UNSPEC);
	if(INVALID_SOCKET == m_ApplicationSock)
	{
		return false;	// Error creating socket
	}
	return true;
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::SetUpSendAddressStructForBroadcast(SOCKADDR_IN &Sockaddr_In, PORT port)
{	
	Sockaddr_In.sin_addr = GetBroadcastAddress();
	Sockaddr_In.sin_family = AF_INET;
	Sockaddr_In.sin_port = htons(port);
}

IN_ADDR CCaseStudy_NotServ_Publish_Client_CPPDlg::GetBroadcastAddress()
{	
	IN_ADDR addr;
	addr.S_un.S_un_b.s_b1 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b2 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b3 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b4 = (unsigned  char) 255;
	return addr;
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::SetUpSendAddress_to_EventNS(DirectoryServiceReply dsr)
{	// The ENS server address and port is retrived from the Directory Service
	m_ENS_Addr.sin_addr.S_un.S_un_b.s_b1 = dsr.a1;
	m_ENS_Addr.sin_addr.S_un.S_un_b.s_b2 = dsr.a2;
	m_ENS_Addr.sin_addr.S_un.S_un_b.s_b3 = dsr.a3;
	m_ENS_Addr.sin_addr.S_un.S_un_b.s_b4 = dsr.a4;
	m_ENS_Addr.sin_family = AF_INET;
	m_ENS_Addr.sin_port = htons(dsr.iPort);
}

IN_ADDR CCaseStudy_NotServ_Publish_Client_CPPDlg::GetLocalHostAddressDetails()
{	
	char szStr[80];
	DWORD lLen = sizeof(szStr);
    GetComputerNameA(szStr,&lLen);
	hostent* pHost;
	pHost = gethostbyname(szStr);
		
	IN_ADDR addr;
	char*  pChar;
	char** ppChar = pHost->h_addr_list;

	if(ppChar != NULL && *ppChar != NULL)  
	{
		pChar = *ppChar; 
		addr.S_un.S_un_b.s_b1 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b2 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b3 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b4 = (unsigned  char) *pChar;
	}
	else
	{
		OnBnClickedCancel();
	}
	return addr;
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::SetUpLocalAddressStruct(SOCKADDR_IN &Sockaddr_In, PORT port)
{	// Initialise the IP address and Port number
	Sockaddr_In.sin_addr = GetLocalHostAddressDetails();
	Sockaddr_In.sin_family = AF_INET;
	Sockaddr_In.sin_port = htons(port);
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::SendResolveMessage() // Broadcast message to Directory server
{
	char cBuffer[50];
	strcpy_s(cBuffer,"RESOLVE:");
	strcat_s(cBuffer,EVENT_NOTIFICATION_SERVER_NAME);
	int iBytesSent;

	iBytesSent = sendto(m_ResolveSock, (char *) cBuffer, sizeof(cBuffer), 0,
							(const sockaddr*)&m_ResolveAddr, sizeof(sockaddr));
	if(SOCKET_ERROR == iBytesSent)
	{
		m_pEventLogList->AddString("sendto (Directory Service) failed");
	}
	m_pEventLogList->AddString("Resolve message sent to Directory Service");
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::Send_PublishMessage_to_EventNotificationServer(char* sEventType, char* EventValue)
{
	int iBytesSent;
	CString cStr;
	if(true == m_bConnected)
	{
		EventNotificationService_Message_PDU ENS_Message;
		ENS_Message.iMessageType = EventNotificationService_Message_Type::PublishEvent;
		strcpy_s(ENS_Message.cEventType,sEventType);
		strcpy_s(ENS_Message.cEventValue,EventValue);
		ENS_Message.SendersAddress = m_LocalAddr;

		iBytesSent = send(m_ApplicationSock, (char *) &ENS_Message, sizeof(EventNotificationService_Message_PDU), 0);
		if(0 < iBytesSent)
		{
			cStr = "Published Event:";
			cStr += sEventType;
			cStr += " Value:";
			cStr += EventValue;
			m_pEventLogList->AddString(cStr);
		}
	}
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::InitiateTimer(UINT iEvent)
{	
	m_nTimer = SetTimer((UINT_PTR) iEvent, 250, 0);	// Frequency = 4HZ
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::StopTimer(UINT iEvent)
{
	KillTimer((UINT_PTR) iEvent);
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::OnTimer(UINT nIDEvent) 
{	
	EventNotificationService_Message_PDU Message_PDU;
	DirectoryServiceReply Message_DirectoryServiceReply;
	int iBytesRecd;
	int iError;
	CString csStr;
	switch(nIDEvent) {
		case Client_WaitForAppServerReply_Event:

			if(true == m_bConnected)
			{
				iBytesRecd = recv(m_ApplicationSock, (char *) &Message_PDU, sizeof(EventNotificationService_Message_PDU), 0);
				if(SOCKET_ERROR == iBytesRecd)
				{
					iError = WSAGetLastError();
					// If no packet is queued for the socket the operation would block.
					if(iError != WSAEWOULDBLOCK)
					{	// WSAEWOULDBLOCK is handled silently, orther errors are reported for diagnostic purposes
						m_pEventLogList->AddString("Receive failed");
						return;
					}	
				}
			}
			break;
		case Client_WaitForDirectoryServiceReply_Event:
			iBytesRecd = recvfrom(m_ResolveSock, (char *)& Message_DirectoryServiceReply, sizeof(DirectoryServiceReply), 0, NULL, NULL);
			if(iBytesRecd > 0)
			{
				StopTimer(Client_WaitForDirectoryServiceReply_Event);
				if(iBytesRecd == sizeof(DirectoryServiceReply)) // expected reply length
				{	
					closesocket(m_ResolveSock); // Close the socket as soon as directory service reply received so to not hog the bound port
												// This allows other clients to run at the same host computer - very useful for development and testing purposes

					if(Message_DirectoryServiceReply.iPort != 0)	// iPort of 0 means App Service was not known
					{   // ONLY update controls if the reply was valid
						csStr.Format("Resolve reply received %d.%d.%d.%d:%d", Message_DirectoryServiceReply.a1,
											Message_DirectoryServiceReply.a2, Message_DirectoryServiceReply.a3,
											Message_DirectoryServiceReply.a4, Message_DirectoryServiceReply.iPort);
						m_pEventLogList->AddString(csStr.GetString());
						
						if(false == m_bConnected)
						{
							SetUpSendAddress_to_EventNS(Message_DirectoryServiceReply);
							ConnectToENS();
						}
					}
				}
			}
		break;
	}
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::ConnectToENS()
{   // Initiate TCP connection with the ENS server
	int iError = connect(m_ApplicationSock, (const SOCKADDR FAR*)&m_ENS_Addr, sizeof(m_ENS_Addr));
	if(SOCKET_ERROR == iError)
	{
		iError = WSAGetLastError();
		// If no packet is queued for the socket the operation would block.
		if(iError != WSAEWOULDBLOCK)
		{	// WSAEWOULDBLOCK is handled silently, orther errors are reported for diagnostic purposes
			m_pEventLogList->AddString("Connect failed !\n\nIs server address correct ?\nIs port correct ?\nIs server running ?");
			closesocket(m_ApplicationSock);		// Close the socket
			return;
		}	
		return;
	}
	m_pEventLogList->AddString("Connected to ENS server");
	m_bConnected = true;
	m_pTemperature_Slider->EnableWindow(true);
	m_pHumiditySlider->EnableWindow(true);
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::OnBnClickedCancel()
{
	closesocket(m_ResolveSock);
	closesocket(m_ApplicationSock);
	CDialog::OnCancel();
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::OnBnClicked_ClearEventLog_Button()
{
	m_pEventLogList->ResetContent();
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::OnNMCustomdrawSlider(NMHDR *pNMHDR, LRESULT *pResult)
{
	CString sStrValue;
	if(true == m_bConnected)
	{
		int iTemperatureCurrent = m_pTemperature_Slider->GetPos();
		if(m_iTemperatureValue_Previous != iTemperatureCurrent)
		{
			m_iTemperatureValue_Previous = iTemperatureCurrent;
			sStrValue.Format("%d",iTemperatureCurrent);
			Send_PublishMessage_to_EventNotificationServer("TemperatureChange",sStrValue.GetBuffer());
			Sleep(200); // Limit rate of sending publish messages when slider is moved quickly
		}
	}	
}

void CCaseStudy_NotServ_Publish_Client_CPPDlg::OnNMCustomdraw_Humidity_Slider(NMHDR *pNMHDR, LRESULT *pResult)
{
	CString sStrValue;
	if(true == m_bConnected)
	{
		int iHumidityCurrent = m_pHumiditySlider->GetPos();
		if(m_iHumidityValue_Previous != iHumidityCurrent)
		{
			m_iHumidityValue_Previous = iHumidityCurrent;
			sStrValue.Format("%d",iHumidityCurrent);
			Send_PublishMessage_to_EventNotificationServer("HumidityChange",sStrValue.GetBuffer());
			Sleep(200); // Limit rate of sending publish messages when slider is moved quickly
		}
	}	
}
