/*	File			CaseStudy_NotificationService_Server_CPPDlg.cpp
	Purpose			Notification Service Server
	Author			Richard Anthony	(R.J.Anthony@gre.ac.uk)
	Date			September 2014
*/

#include "stdafx.h"

#include "CaseStudy_NotificationService_Server_CPP.h"
#include "CaseStudy_NotificationService_Server_CPPDlg.h"
#include "string.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CCaseStudy_NotificationService_Server_CPPDlg::CCaseStudy_NotificationService_Server_CPPDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCaseStudy_NotificationService_Server_CPPDlg::IDD, pParent)
{
	m_csAppName = EVENT_NOTIFICATION_SERVER_NAME;
	bRegisterAlreadyDone = false;
	
	bool bResult = CreateRegisterSocket();
	if(false == bResult)
	{
		OnBnClickedCancel();
	}
	
	bResult = CreateListenSocket();
	if(false == bResult)
	{
		OnBnClickedCancel();
	}

	SetUpSendAddressStructForBroadcast(m_RegisterAddr, To_DIRECTORY_SERVICE_PORT);
	m_iNumberofClientRequests = 0;	
	
	// Initialise the connections array
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		m_ConnectionArray[i].bInUse = false;
	}	
	// Initialise the EventType_List array
	for(int i = 0; i < MAX_EVENT_TYPES; i++)
	{	
		m_EventType_List[i].bInUse_SubscriptionExists = false;
		m_EventType_List[i].bInUse_ValueValid = false;
		m_EventType_List[i].pSubscriberList_Head = (SubscriberListItem*) NULL;
	}	
}

CCaseStudy_NotificationService_Server_CPPDlg::~CCaseStudy_NotificationService_Server_CPPDlg()
{
	closesocket(m_ListenSock);
	closesocket(m_RegisterSock);
}

void CCaseStudy_NotificationService_Server_CPPDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	m_pEventLogList = (CListBox*) GetDlgItem(IDC_EventLog_LIST);
	m_pEventTypeList = (CListBox*) GetDlgItem(IDC_EVENT_TYPE_LIST);
	Activate_ENS();
	DisplayEventTypeList();	// Refresh the list of known event types
}

BEGIN_MESSAGE_MAP(CCaseStudy_NotificationService_Server_CPPDlg, CDialog)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDCANCEL, &CCaseStudy_NotificationService_Server_CPPDlg::OnBnClickedCancel)
	ON_BN_CLICKED(ClearLog_BUTTON, &CCaseStudy_NotificationService_Server_CPPDlg::OnBnClickedButton)
END_MESSAGE_MAP()

bool CCaseStudy_NotificationService_Server_CPPDlg::CreateRegisterSocket()
{   // UDP, non-blocking, broadcast
	m_RegisterSock = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
	if(INVALID_SOCKET == m_RegisterSock)
	{
		return false;	// Error creating socket
	}

	unsigned long lNonBlocking = 1;
	int iError = ioctlsocket(m_RegisterSock, FIONBIO, &lNonBlocking);
	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting non-blocking behaviour
	}

	// Turn on Broadcast mode
	char cOpt[2];
	cOpt[0] = 1; // true
	cOpt[1] = 0; // null terminate the option array
	iError = setsockopt(m_RegisterSock, SOL_SOCKET, SO_BROADCAST, cOpt, sizeof(cOpt)); 
  	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting broadcast option
	}
	return true;
}

bool CCaseStudy_NotificationService_Server_CPPDlg::CreateListenSocket()
{   // TCP, non-blocking
	m_ListenSock = socket(AF_INET, SOCK_STREAM, PF_UNSPEC);
	if(INVALID_SOCKET == m_ListenSock)
	{
		return false;	// Error creating socket
	}

	unsigned long lNonBlocking = 1;
	int iError = ioctlsocket(m_ListenSock, FIONBIO, &lNonBlocking);
	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting non-blocking behaviour
	}
	return true;
}

void CCaseStudy_NotificationService_Server_CPPDlg::SetUpSendAddressStructForBroadcast(SOCKADDR_IN &Sockaddr_In, PORT port)
{	
	Sockaddr_In.sin_addr = GetBroadcastAddress();
	Sockaddr_In.sin_family = AF_INET;
	Sockaddr_In.sin_port = htons(port);
}

IN_ADDR CCaseStudy_NotificationService_Server_CPPDlg::GetBroadcastAddress()
{	
	IN_ADDR addr;
	addr.S_un.S_un_b.s_b1 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b2 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b3 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b4 = (unsigned  char) 255;
	return addr;
}

IN_ADDR CCaseStudy_NotificationService_Server_CPPDlg::GetLocalHostAddressDetails()
{	
	char szStr[80];
	DWORD lLen = sizeof(szStr);
    GetComputerNameA(szStr,&lLen);
	hostent* pHost;
	pHost = gethostbyname(szStr);
		
	IN_ADDR addr;
	char*  pChar;
	char** ppChar = pHost->h_addr_list;

	if(ppChar != NULL && *ppChar != NULL)  
	{
		pChar = *ppChar; 
		addr.S_un.S_un_b.s_b1 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b2 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b3 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b4 = (unsigned  char) *pChar;
	}
	else
	{
		OnBnClickedCancel();
	}
	return addr;
}

void CCaseStudy_NotificationService_Server_CPPDlg::SetUpLocalAddressStruct(SOCKADDR_IN &Sockaddr_In, PORT port)
{	// Initialise the IP address and Port number
	Sockaddr_In.sin_addr = GetLocalHostAddressDetails();
	Sockaddr_In.sin_family = AF_INET;
	Sockaddr_In.sin_port = htons(port);
}

void CCaseStudy_NotificationService_Server_CPPDlg::SendRegisterMessage() // Broadcast message to Directory Server
{
	char cBuffer[50];
	char cPort[6];
	_itoa_s(ENS_APPLICATION_PORT, cPort, 10);
	strcpy_s(cBuffer,"REGISTER:");
	strcat_s(cBuffer,m_csAppName.GetString());
	strcat_s(cBuffer,":");
	strcat_s(cBuffer,cPort);
	int iBytesSent;

	iBytesSent = sendto(m_RegisterSock, (char *) cBuffer, sizeof(cBuffer), 0,
							(const sockaddr*)&m_RegisterAddr, sizeof(sockaddr));
	if(SOCKET_ERROR == iBytesSent)
	{
		m_pEventLogList->AddString("sendto (Directory Service) failed");
	}
	if(false == bRegisterAlreadyDone)
	{
		m_pEventLogList->AddString("Registered with Directory Service");
		bRegisterAlreadyDone = true;
	}
}

void CCaseStudy_NotificationService_Server_CPPDlg::InitiateRegistrationTimer()
{	
	m_nTimer = SetTimer(1, 10000, 0);	// 10 second interval between auto-registration activity
}

void CCaseStudy_NotificationService_Server_CPPDlg::InitiateCommunicationTimer()
{	
	m_nTimer = SetTimer(2, 50, 0);	// 50 milli-second interval for instances of accept and recv activity
}

void CCaseStudy_NotificationService_Server_CPPDlg::OnTimer(UINT nIDEvent) 
{	
	if(nIDEvent == 1) // Registration interval has expired
	{
		SendRegisterMessage();	// Periodic auto-registration
		return;
	}

	if(nIDEvent == 2) // Communication check interval has expired
	{
		Accept();	// Deal with any pending connection requests

		for(int iIndex = 0; iIndex < MAX_CONNECTIONS; iIndex++)	// Deal with any pending messages in the buffer
		{	
			if(m_ConnectionArray[iIndex].bInUse == true)
			{
				Receive(iIndex);	// For each valid listen-socket DoReceive()
			}
		}
	}
}

void CCaseStudy_NotificationService_Server_CPPDlg::Receive(int iConnection)
{		
	EventNotificationService_Message_PDU Message;
	int iBytesRecd = recv(m_ConnectionArray[iConnection].m_iConnectSock, (char *) &Message, sizeof(EventNotificationService_Message_PDU), 0);
	if(SOCKET_ERROR == iBytesRecd)
	{
		int iError = WSAGetLastError();
		// If no packet is queued for the socket the operation would block.
		if(iError != WSAEWOULDBLOCK)
		{	// WSAEWOULDBLOCK is handled silently, orther errors are reported for diagnostic purposes
			m_pEventLogList->AddString("Receive failed");
			ConnectionLost(iConnection, iError);	
			return;
		}	
	}
	else
	{
		if(iBytesRecd == 0)
		{
			return;
		}

		CString csStr;
		int iEventTypeList_Index;
		switch(Message.iMessageType)
		{
			case SubscribeToEvent:
				csStr.Format("SubscribeToEvent received: %s",Message.cEventType);
				m_pEventLogList->AddString(csStr.GetString());
				AddEventType(Message, iConnection);
				DisplayEventTypeList();	// Refresh the list of known event types
				break;
			case PublishEvent:
				csStr.Format("PublishEvent received: %s %s",Message.cEventType, Message.cEventValue);
				m_pEventLogList->AddString(csStr.GetString());
				iEventTypeList_Index = AddEventType(Message, iConnection);
				DisplayEventTypeList();	// Refresh the list of known event types
				PushEventToRegisteredSubscribers(iEventTypeList_Index);
				break;
			case UnSubscribeToEvent:
				csStr.Format("UnSubscribeToEvent received: %s",Message.cEventType);
				m_pEventLogList->AddString(csStr.GetString());
				RemoveSubscription(Message, iConnection);
				DisplayEventTypeList();	// Refresh the list of known event types
				break;
		}
	}
}

int CCaseStudy_NotificationService_Server_CPPDlg::GetIndexOfEvent(EventNotificationService_Message_PDU Message, bool bCreateNewIfNotExist)
{
	int iIndex;
	// Check if the Event Type is already known
	for(iIndex = 0; iIndex < MAX_EVENT_TYPES; iIndex++)
	{	
		if(true == m_EventType_List[iIndex].bInUse_SubscriptionExists || true == m_EventType_List[iIndex].bInUse_ValueValid)
		{
			if(0 == strcmp(m_EventType_List[iIndex].cEventType,Message.cEventType))
			{	
				m_pEventLogList->AddString("EventType already known");
				return iIndex;
			}
		}
	}	
	
	if(false == bCreateNewIfNotExist)
	{   // Do not create a new entry if the EventType was not found (for UnSubscribe)
		m_pEventLogList->AddString("EventType not found");
		return -1; // Event entry was not found
	}
		
	// Find an unused m_EventType_List entry (for Subscribe, Publish)
	for(iIndex = 0; iIndex < MAX_EVENT_TYPES; iIndex++)
	{	
		if(false == m_EventType_List[iIndex].bInUse_SubscriptionExists && false == m_EventType_List[iIndex].bInUse_ValueValid)
		{
			m_pEventLogList->AddString("Vacant EventType entry found");
			// Add the new event type in the vacent list entry
			strcpy_s(m_EventType_List[iIndex].cEventType, Message.cEventType); 
			return iIndex;
		}
	}
	m_pEventLogList->AddString("EventType list FULL !");
	return -1;
}

int CCaseStudy_NotificationService_Server_CPPDlg::AddEventType(EventNotificationService_Message_PDU Message, int iConnection)
{
	int iIndex;
	switch(Message.iMessageType) {
		case EventNotificationService_Message_Type::SubscribeToEvent:
			iIndex = GetIndexOfEvent(Message, true);  // Returns index of event (creates a new event if the EventType does not already exist)
			m_EventType_List[iIndex].bInUse_SubscriptionExists = true;	
			AddSubscriber_to_SubscriberList(iIndex, iConnection);	// Add subscriber details to subscriber list for event type
			if(true == m_EventType_List[iIndex].bInUse_ValueValid)
			{   // The event had already been published, so push the current value out to the new subscriber immediately
				Send_EventUpdate_Message(iIndex, iConnection);
			}
			return iIndex;
		case EventNotificationService_Message_Type::PublishEvent:
			iIndex = GetIndexOfEvent(Message, true);  // Returns index of event (creates a new event if the EventType does not already exist)
			m_EventType_List[iIndex].bInUse_ValueValid = true;
			if(0 == Message.cEventValue[0])
			{   // If EventValue string is blank set it to "0"
				strcpy_s(Message.cEventValue, "0");
			}
			strcpy_s(m_EventType_List[iIndex].cEventValue, Message.cEventValue);
			m_EventType_List[iIndex].PublisherAddress = Message.SendersAddress;
			return iIndex;
	}
	return -1;
}

void CCaseStudy_NotificationService_Server_CPPDlg::AddSubscriber_to_SubscriberList(int iIndex, int iConnection)
{
	// Iterate through the list, each time checking to see if the current subscriber is already listed
	SubscriberListItem* pCurrent = m_EventType_List[iIndex].pSubscriberList_Head;
	while(NULL != pCurrent)
	{
		if(pCurrent->iConnectionIndex == iConnection)
		{
			return;   // Subscriber already in the list for the particular EventType
		}
		pCurrent = pCurrent->pSubscriberList_Next;
	}
	// Subscriber not found in list, add at head of list
	SubscriberListItem* pNew = new SubscriberListItem;
	pNew->iConnectionIndex = iConnection;
	pNew->pSubscriberList_Next = m_EventType_List[iIndex].pSubscriberList_Head;
	m_EventType_List[iIndex].pSubscriberList_Head = pNew;
}

void CCaseStudy_NotificationService_Server_CPPDlg::PushEventToRegisteredSubscribers(int iEventTypeList_Index)
{
	// Iterate through the list, send an Event update message to each subscriber in the list
	SubscriberListItem* pCurrent = m_EventType_List[iEventTypeList_Index].pSubscriberList_Head;
	while(NULL != pCurrent)
	{
		Send_EventUpdate_Message(iEventTypeList_Index, pCurrent->iConnectionIndex);
		pCurrent = pCurrent->pSubscriberList_Next;
	}
}

void CCaseStudy_NotificationService_Server_CPPDlg::Send_EventUpdate_Message(int iEventTypeList_Index, int iConnectionIndex)
{
	EventNotificationService_Message_PDU ENS_Message;
	ENS_Message.iMessageType = EventNotificationService_Message_Type::EventUpdate;
	strcpy_s(ENS_Message.cEventType, m_EventType_List[iEventTypeList_Index].cEventType);
	strcpy_s(ENS_Message.cEventValue, m_EventType_List[iEventTypeList_Index].cEventValue);
	ENS_Message.SendersAddress = m_LocalAddr;
	SendMessageToClient(ENS_Message, iConnectionIndex);
}

void CCaseStudy_NotificationService_Server_CPPDlg::SendMessageToClient(EventNotificationService_Message_PDU ENS_Message, int iConnectionIndex)
{
	int iBytesSent;
	iBytesSent = send(m_ConnectionArray[iConnectionIndex].m_iConnectSock, (char *) &ENS_Message, sizeof(EventNotificationService_Message_PDU), 0);
	if(0 < iBytesSent)
	{
		m_pEventLogList->AddString("Message sent to ENS client");
	}
}

void CCaseStudy_NotificationService_Server_CPPDlg::RemoveSubscription(EventNotificationService_Message_PDU Message, int iConnection)
{
	// Find the relevent entry in linked list
	int iIndex;
	iIndex = GetIndexOfEvent(Message, false);  // Returns index of event (does not create a new event if the EventType does not already exist)
	if(-1 == iIndex)
	{
		return;	// EventType entry not found in array
	}
	RemoveSubscriber_from_SubscriberList(iIndex, iConnection);	// Remove subscriber details from subscriber list for event type
	
	if(NULL == m_EventType_List[iIndex].pSubscriberList_Head)
	{	// If subscriber list now emply, set subscribed flag  to false
		m_EventType_List[iIndex].bInUse_SubscriptionExists = false;
	}

	// If both flags are now false (the bInUse_SubscriptionExists flag and the bInUse_ValueValid flag) the EventType
	// is essentially removed, as its array entry will be re-used when a new EventType is created
}

void CCaseStudy_NotificationService_Server_CPPDlg::RemoveSubscriber_from_SubscriberList(int iIndex, int iConnection)
{
	// Iterate through the list, each time checking to see if the current subscriber is listed
	SubscriberListItem* pDelete;
	if(NULL == m_EventType_List[iIndex].pSubscriberList_Head)
	{
		return;
	}
	
	if(m_EventType_List[iIndex].pSubscriberList_Head->iConnectionIndex == iConnection)
	{   // The item to be deleted is the head of the list, remove it and then delete it
		pDelete = m_EventType_List[iIndex].pSubscriberList_Head;
		m_EventType_List[iIndex].pSubscriberList_Head = m_EventType_List[iIndex].pSubscriberList_Head->pSubscriberList_Next;
		delete pDelete;
		return;
	}

	// The item to be deleted is not the head of the list, iterate to find it, then remove it and delete it
	// The technique used (to look at the curent item's 'next' pointer object, rather than the current object)
	// only manipulates the next pointers within established objects. It does not insert new pointers into
	// the chain which would disconnect part of the chain from the head.
	SubscriberListItem* pCurrent = m_EventType_List[iIndex].pSubscriberList_Head;
	while(NULL != pCurrent && NULL != pCurrent->pSubscriberList_Next)
	{
		if(pCurrent->pSubscriberList_Next->iConnectionIndex == iConnection)
		{
			pDelete = pCurrent->pSubscriberList_Next;
			pCurrent->pSubscriberList_Next = pCurrent->pSubscriberList_Next->pSubscriberList_Next;
			delete pDelete;
			return;
		}
		pCurrent = pCurrent->pSubscriberList_Next;	// Move along list for next iteration
	}
}

void CCaseStudy_NotificationService_Server_CPPDlg::Accept()
{
	CString csStr;
	int iIndex = GetIndexOfUnusedConnection();
	if(iIndex == -1)
	{
		m_pEventLogList->AddString("Accept refused - Maximum number of Connections reached");
		return;
	}
	
	int iRemoteAddrLen = sizeof(SOCKADDR_IN);
	int iError;
	bool bResult = false;
	
	m_ConnectionArray[iIndex].m_iConnectSock = accept(m_ListenSock, (SOCKADDR FAR*)&m_ConnectionArray[iIndex].m_ConnectSockAddr, &iRemoteAddrLen);
	if(INVALID_SOCKET == m_ConnectionArray[iIndex].m_iConnectSock)
	{
		iError = WSAGetLastError();
		// If no connection request is queued for the socket the operation would block.
		if(iError != WSAEWOULDBLOCK)
		{	
			m_pEventLogList->AddString("Accept failed");
			return;
		}
	}
	else
	{
		// Set the newly-created connection socket to Non-Blocking
		u_long lBlocking = 1; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
		iError = ioctlsocket(m_ConnectionArray[iIndex].m_iConnectSock, FIONBIO, &lBlocking); // Set non-blocking socket behaviour
		if(NO_ERROR != iError)
		{
			m_pEventLogList->AddString("Could not set non-Blocking behaviour for connection socket");
			return;	
		}

		m_ConnectionArray[iIndex].bInUse = true;
		csStr.Format("Client %d.%d.%d.%d Connected", 
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1,
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2, 
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3,
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4);
		m_pEventLogList->AddString(csStr.GetString());
	}
}

void CCaseStudy_NotificationService_Server_CPPDlg::Activate_ENS()
{
	SetUpLocalAddressStruct(m_LocalAddr, ENS_APPLICATION_PORT);

	// Bind to the Application port that has been advertised via the Directory Service
	int iResult = bind(m_ListenSock, (const SOCKADDR FAR*)&m_LocalAddr, sizeof(m_LocalAddr));
	if(SOCKET_ERROR == iResult)
	{
		m_pEventLogList->AddString("Bind failed");
		OnBnClickedCancel();
	}

	// Listen - for TCP connection requests from clients
	iResult = listen(m_ListenSock, SOMAXCONN); 
	if(SOCKET_ERROR == iResult)
	{
		OnBnClickedCancel();
	}

	SendRegisterMessage();		// Auto-register with Directory service immediately
	InitiateRegistrationTimer();// Auto-register with Directory service periodically
	InitiateCommunicationTimer(); // Deal with connection requests and received messages on short-periodic basis
}

int CCaseStudy_NotificationService_Server_CPPDlg::GetIndexOfUnusedConnection()
{
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == false)
		{
			return i;
		}
	}
	return -1;	// Signal that all connections are in use
}

void CCaseStudy_NotificationService_Server_CPPDlg::DisplayEventTypeList()
{
	while(0 < m_pEventTypeList->GetCount())  // Clear the Listbox
	{
		m_pEventTypeList->DeleteString(0);
	}
	
	for(int i = 0; i < MAX_EVENT_TYPES; i++)
	{	
		if(true == m_EventType_List[i].bInUse_SubscriptionExists || true == m_EventType_List[i].bInUse_ValueValid)
		{
			m_pEventTypeList->AddString(m_EventType_List[i].cEventType);
		}
	}	
}

void CCaseStudy_NotificationService_Server_CPPDlg::OnBnClickedCancel()
{
	closesocket(m_RegisterSock);
	closesocket(m_ListenSock);
	CDialog::OnCancel();
}

void CCaseStudy_NotificationService_Server_CPPDlg::ConnectionLost(int iConnection, int iError)			
{
	if(WSAECONNABORTED == iError || WSAECONNRESET == iError ||
		WSAETIMEDOUT == iError || WSAENETRESET == iError ||
		WSAESHUTDOWN == iError)
	{
		CloseConnection(iConnection);
	}
}

void CCaseStudy_NotificationService_Server_CPPDlg::CloseConnection(int iConnection)
{	
	shutdown(m_ConnectionArray[iConnection].m_iConnectSock,SD_BOTH);	// Close the connection
	closesocket(m_ConnectionArray[iConnection].m_iConnectSock);			// Close the socket
	m_ConnectionArray[iConnection].bInUse = false;

	CString csStr;
	csStr.Format("%d.%d.%d.%d Disconnected", 
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1,
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2, 
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3,
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4);
	m_pEventLogList->AddString(csStr.GetString());

}

void CCaseStudy_NotificationService_Server_CPPDlg::CloseAll()
{
	for(int i = 0; i < MAX_CONNECTIONS; i++)	// Close active connections
	{	
		if(m_ConnectionArray[i].bInUse == true)
		{
			CloseConnection(i);
		}
	}
	closesocket(m_ListenSock);		// Close the listen socket
}

void CCaseStudy_NotificationService_Server_CPPDlg::OnBnClickedButton()
{
	m_pEventLogList->ResetContent();
}
