/*	File			CaseStudy_NotificationService_Server_CPPDlg.h
	Purpose			Notification Service Server
	Author			Richard Anthony	(R.J.Anthony@gre.ac.uk)
	Date			September 2014
*/

#pragma once
#include "NotificationServiceDeffinitions.h"
#include "afxwin.h"
#define MAX_CONNECTIONS 30
#define MAX_EVENT_TYPES 30

struct Connection {
	SOCKET m_iConnectSock;
	SOCKADDR_IN m_ConnectSockAddr;
	bool bInUse;
};

struct SubscriberListItem {
	int iConnectionIndex; // Index into connection array to identify the subscriber
	SubscriberListItem* pSubscriberList_Next;
};

struct Event_Type {     // An event type is dynamically created whenever a publisher publishes a new event type or a consumer subscribes to a new event type
						// this is totally run-time configured, the designer does not know anything about the event types that may exist
						// Components are decoupled at the application level as well as at the communication level:
						//		- a producer can publish a new event type regardless of whether there is any consumer for that event-type
						//		- a consumer can subscribe to a new event type regardless of whether there is any producer for that event-type
	char cEventType[MAX_EVENT_TYPE_NAME_LENGTH];
	char cEventValue[MAX_EVENT_VALUE_LENGTH];
	SubscriberListItem* pSubscriberList_Head;	// Linked list of subscribers
	SOCKADDR_IN PublisherAddress;	// IP address and Port number of most recent publisher of this event type
	bool bInUse_ValueValid;			// Has the event been published
	bool bInUse_SubscriptionExists; // Is there at least one consumer waiting for this event
};


class CCaseStudy_NotificationService_Server_CPPDlg : public CDialog
{
public:
	CCaseStudy_NotificationService_Server_CPPDlg(CWnd* pParent = NULL);	// standard constructor
	virtual ~CCaseStudy_NotificationService_Server_CPPDlg();

	enum { IDD = IDD_CASESTUDY_NOTIFICATIONSERVICE_SERVER_CPP_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);

	DECLARE_MESSAGE_MAP()

public:
	void Activate_ENS();
	bool CreateRegisterSocket();
	bool CreateListenSocket();
	void SetUpSendAddressStructForBroadcast(SOCKADDR_IN &Sockaddr_In, PORT port);
	IN_ADDR GetBroadcastAddress();
	IN_ADDR GetLocalHostAddressDetails();
	void SetUpLocalAddressStruct(SOCKADDR_IN &Sockaddr_In, PORT port);
	void SendRegisterMessage();
	void InitiateRegistrationTimer();
	void InitiateCommunicationTimer();
	void OnTimer(UINT nIDEvent); 
	int GetIndexOfUnusedConnection();
	void Accept();
	void Receive(int iConnection);

	int GetIndexOfEvent(EventNotificationService_Message_PDU Message, bool bCreateNewIfNotExist);
	int AddEventType(EventNotificationService_Message_PDU Message, int iConnection);
	void AddSubscriber_to_SubscriberList(int iIndex, int iConnection);
	void PushEventToRegisteredSubscribers(int iEventTypeList_Index);
	void Send_EventUpdate_Message(int iEventTypeList_Index, int iConnectionIndex);
	void SendMessageToClient(EventNotificationService_Message_PDU ENS_Message, int iConnectionIndex);
	void RemoveSubscription(EventNotificationService_Message_PDU Message, int iConnection);
	void RemoveSubscriber_from_SubscriberList(int iIndex, int iConnection);

	void DisplayEventTypeList();
	void ConnectionLost(int iConnection, int iError);
	void CloseConnection(int iConnection);
	void CloseAll();

private:

	CString m_csAppName;
	bool bRegisterAlreadyDone;
	int m_iNumberofClientRequests;

	UINT_PTR m_nTimer;			// Needed because receives are non-blocking

	SOCKET m_RegisterSock;		// Socket for sending 'Register' messages to directory service
	SOCKET m_ListenSock;		// Socket for communication with clients
	SOCKADDR_IN m_RecvAddr;		// holds From-address (passed back from Recvfrom() )
	SOCKADDR_IN m_LocalAddr;	// Address for Receiving client requests
	SOCKADDR_IN m_RegisterAddr;	// for Broadcasting 'register' messages to the Directory server
	Connection m_ConnectionArray[MAX_CONNECTIONS];
	Event_Type m_EventType_List[MAX_EVENT_TYPES];

	CListBox* m_pEventLogList;
	afx_msg void OnBnClickedCancel();
	CListBox* m_pEventTypeList;

public:
	afx_msg void OnBnClickedButton();
};
