
/*	File			CaseStudy_TimeServiceClient_GUIDlg.cpp
	Purpose			Network Time Protocol (NTP) Client header code
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			September 2014
*/

#include "stdafx.h"
#include "CaseStudy_TimeServiceClient_GUI.h"
#include "CaseStudy_TimeServiceClient_GUIDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CCaseStudy_TimeServiceClient_GUIDlg::CCaseStudy_TimeServiceClient_GUIDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCaseStudy_TimeServiceClient_GUIDlg::IDD, pParent)
{
	m_bNTP_Client_Started = false;
	m_iNumRequestsSent = 0;
	m_iNumResponsesReceived = 0;
}

CCaseStudy_TimeServiceClient_GUIDlg::~CCaseStudy_TimeServiceClient_GUIDlg()
{	
	closesocket(m_TimeService_Socket);
}

void CCaseStudy_TimeServiceClient_GUIDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	
	CString csStr;
	m_pUTC_Time = (CEdit*) GetDlgItem(IDC_UTC_Time_EDIT);
	m_pNTP_StartButton = (CButton*) GetDlgItem(IDC_NTP_Start_BUTTON);
	m_pNTP_URL_List = (CListBox*) GetDlgItem(IDC_NTP_SERVER_URL_LIST);
	m_pNTP_Location_List = (CListBox*) GetDlgItem(IDC_NTP_SERVER_Location_LIST);
	m_pNTP_Location_List->EnableWindow(false);
	Populate_NTP_Server_List();
	m_pNTP_Address_1 = (CEdit*) GetDlgItem(IDC_NTP_ADD_1);
	m_pNTP_Address_2 = (CEdit*) GetDlgItem(IDC_NTP_ADD_2);
	m_pNTP_Address_3 = (CEdit*) GetDlgItem(IDC_NTP_ADD_3);
	m_pNTP_Address_4 = (CEdit*) GetDlgItem(IDC_NTP_ADD_4);
	m_pNTP_PORT = (CEdit*) GetDlgItem(IDC_NTP_PORT);
	m_pSelectedURL = (CEdit*) GetDlgItem(IDC_Selected_URL_EDIT);
	m_pSecondsSince1900 = (CEdit*) GetDlgItem(IDC_SecondsSince1900_EDIT);
	m_pNumNTPRequestsSent = (CEdit*) GetDlgItem(IDC_NUM_NTP_REQUESTS_EDIT);
	m_pNumNTPResponsesReceived = (CEdit*) GetDlgItem(IDC_NUM_NTP_RESPONSES_EDIT);
	m_pNTP_PORT->EnableWindow(false);
	m_pSelectedURL->EnableWindow(false);
	m_pNumNTPRequestsSent->EnableWindow(false);
	m_pNumNTPResponsesReceived->EnableWindow(false);
	m_pNTP_URL_List->SetCurSel(0);		// Select first entry in the list
	OnLbnSelchangeNtpServerUrlList();	// Highlight selection across all three list boxes, set the server address structure contents and populate address edit boxes
}

BEGIN_MESSAGE_MAP(CCaseStudy_TimeServiceClient_GUIDlg, CDialog)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_NTP_Start_BUTTON, &CCaseStudy_TimeServiceClient_GUIDlg::OnBnClickedNtpStartButton)
	ON_LBN_SELCHANGE(IDC_NTP_SERVER_URL_LIST, &CCaseStudy_TimeServiceClient_GUIDlg::OnLbnSelchangeNtpServerUrlList)
	ON_BN_CLICKED(IDCANCEL, &CCaseStudy_TimeServiceClient_GUIDlg::OnBnClickedCancel)
END_MESSAGE_MAP()

void CCaseStudy_TimeServiceClient_GUIDlg::OnBnClickedNtpStartButton()
{
	if(false == m_bNTP_Client_Started)
	{
		m_pNTP_StartButton->SetWindowText("STOP NTP requests");
		m_bNTP_Client_Started = true;
		m_pNTP_Address_1->EnableWindow(false);
		m_pNTP_Address_2->EnableWindow(false);
		m_pNTP_Address_3->EnableWindow(false);
		m_pNTP_Address_4->EnableWindow(false);
		m_pNTP_URL_List->EnableWindow(false);	
		m_iNumRequestsSent = 0;
		m_iNumResponsesReceived = 0;
		UpdateStatisticsDisplay();
		StartPeriodic_NTP_requests();
	}
	else
	{
		m_pNTP_StartButton->SetWindowText("START NTP requests");
		m_bNTP_Client_Started = false;
		m_pNTP_Address_1->EnableWindow(true);
		m_pNTP_Address_2->EnableWindow(true);
		m_pNTP_Address_3->EnableWindow(true);
		m_pNTP_Address_4->EnableWindow(true);
		m_pNTP_URL_List->EnableWindow(true);
		StopPeriodic_NTP_requests();
	}
}

void CCaseStudy_TimeServiceClient_GUIDlg::StartPeriodic_NTP_requests()
{
	// Create a socket
	bool bSuccess = CreateSocket(&m_TimeService_Socket);
	if(false == bSuccess)
	{
		closesocket(m_TimeService_Socket);
	}

	// Set Socket to Non-Blocking Mode
	bSuccess = SetSocketToNonBlockingMode(&m_TimeService_Socket);
	if(false == bSuccess)
	{
		closesocket(m_TimeService_Socket);
	}
	Send_TimeService_Request();

	InitiateTimer();				// Subsequent NTP requests are made every 10 seconds
}

bool CCaseStudy_TimeServiceClient_GUIDlg::CreateSocket(SOCKET* sock)
{
	*sock = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
	if(INVALID_SOCKET == *sock)
	{
		return false;
	}
	return true;
}
//
bool CCaseStudy_TimeServiceClient_GUIDlg::SetSocketToNonBlockingMode(SOCKET* sock)
{
	unsigned long lNonBlocking = 1;
	int iError = ioctlsocket(*sock, FIONBIO, &lNonBlocking);
	if(SOCKET_ERROR == iError)
	{
		return false;
	}
	return true;
}

void CCaseStudy_TimeServiceClient_GUIDlg::StopPeriodic_NTP_requests()
{
	StopTimer();
	closesocket(m_TimeService_Socket);
}

bool CCaseStudy_TimeServiceClient_GUIDlg::Receive()
{
	// The process inspects its buffer to see if any messages have arrived
	int iBytesRecd = recvfrom(m_TimeService_Socket, (char FAR*)ReceivePacketBuffer, NTP_PACKET_SIZE, 0, NULL, NULL);
	if(SOCKET_ERROR == iBytesRecd)
	{
		return false;
	}
	else
	{	
		// The timestamp starts at byte 40 of the received packet and is four bytes,
		unsigned long secsSince1900 = (ReceivePacketBuffer[40] << 24) + (ReceivePacketBuffer[41] << 16) + (ReceivePacketBuffer[42] << 8) + ReceivePacketBuffer[43];  
		// Unix time starts on Jan 1 1970 (2208988800 seconds)
		const unsigned long seventyYears = 2208988800UL;     
		unsigned long lUnixTime = secsSince1900 - seventyYears;	// subtract seventy years:
		unsigned long lHour = (lUnixTime  % 86400L) / 3600;
		unsigned long lMinute = (lUnixTime  % 3600) / 60;
		unsigned long lSecond = lUnixTime  % 60;
	 	 
		CString csStrUTC_Time;
		csStrUTC_Time.Format("%d", lUnixTime);
		m_pSecondsSince1900->SetWindowTextA(csStrUTC_Time.GetString());
		csStrUTC_Time.Format("%02d:%02d:%02d", lHour, lMinute, lSecond);
		m_pUTC_Time->SetWindowTextA(csStrUTC_Time.GetString());
		// NTP does not indicate information specifcally related to local time zones such as BST or other daylight savings time offsets
		m_iNumResponsesReceived++;
		UpdateStatisticsDisplay();
	}
	return true;
}

void CCaseStudy_TimeServiceClient_GUIDlg::InitiateTimer()
{	
	// Do not make NTP requests faster than once every 4 seconds or NIST will treat it as a DoS attack
	m_nTimer = SetTimer(1, 10000, 0);	// 10 second period for send NTP request
	m_nTimer = SetTimer(2, 200, 0);		// 0.2 second period for listening for received NTP responses
}

void CCaseStudy_TimeServiceClient_GUIDlg::StopTimer()
{
	KillTimer(m_nTimer);
}

void CCaseStudy_TimeServiceClient_GUIDlg::OnTimer(UINT nIDEvent) 
{	
	if(1 == nIDEvent)
	{
		Send_TimeService_Request();
	}	
	if(2 == nIDEvent)
	{
		Receive();
	}
}

bool CCaseStudy_TimeServiceClient_GUIDlg::Send_TimeService_Request()
{
	memset(SendPacketBuffer, 0, NTP_PACKET_SIZE);	// Zero-out entire 48-byte array
	// Initialize values needed to form NTP request
	SendPacketBuffer[0] = 0xE3;	// 0b11100011;   
								// LI bits 7,6			= 3 (Clock not synchronised), 
								// Version bits 5,4,3	= 4 (The current version of NTP)
								// Mode bits 2,1,0		= 3 (Sent by client)

	m_iSendLen = sizeof(SendPacketBuffer);

	int iBytesSent = sendto(m_TimeService_Socket, (char FAR *)SendPacketBuffer, m_iSendLen, 0,
		(const struct sockaddr FAR *)&m_TimeService_SockAddr, sizeof(m_TimeService_SockAddr));
	if(INVALID_SOCKET == iBytesSent)
	{
		return false;
	}
	m_iNumRequestsSent++;
	UpdateStatisticsDisplay();
	return true;
}

in_addr CCaseStudy_TimeServiceClient_GUIDlg::ResolveURLtoIPaddress(CString cStrURL)
{
    DWORD dwError;
    struct hostent *TimeServiceHost;
    struct in_addr HostAddr;
	TimeServiceHost = gethostbyname(cStrURL.GetString() /*sHostName*/);
 
    if(NULL == TimeServiceHost) 
    {   // gethostbyname failed to resove time service domain name
        dwError = WSAGetLastError();
		HostAddr.S_un.S_un_b.s_b1 = 0;
		HostAddr.S_un.S_un_b.s_b2 = 0;
		HostAddr.S_un.S_un_b.s_b3 = 0;
		HostAddr.S_un.S_un_b.s_b4 = 0;
    } 
    else 
    {   // gethostbyname successfully resoved time service domain name
		HostAddr.S_un.S_addr = *(u_long *) TimeServiceHost->h_addr_list[0]; // Get first address from host's list of addresses
	}
	return HostAddr;
}

void CCaseStudy_TimeServiceClient_GUIDlg::SetUp_TimeService_AddressStruct(CString cStrURL)
{
	// Step 1 resolve URL to IP address
	struct in_addr HostAddr = ResolveURLtoIPaddress(cStrURL);
	CString csIPAddressByte;

	m_TimeService_SockAddr.sin_addr.S_un.S_un_b.s_b1 = HostAddr.S_un.S_un_b.s_b1;
	m_TimeService_SockAddr.sin_addr.S_un.S_un_b.s_b2 = HostAddr.S_un.S_un_b.s_b2;
	m_TimeService_SockAddr.sin_addr.S_un.S_un_b.s_b3 = HostAddr.S_un.S_un_b.s_b3;
	m_TimeService_SockAddr.sin_addr.S_un.S_un_b.s_b4 = HostAddr.S_un.S_un_b.s_b4;

	csIPAddressByte.Format("%d",m_TimeService_SockAddr.sin_addr.S_un.S_un_b.s_b1);
	m_pNTP_Address_1->SetWindowText(csIPAddressByte.GetString());
	csIPAddressByte.Format("%d",m_TimeService_SockAddr.sin_addr.S_un.S_un_b.s_b2);
	m_pNTP_Address_2->SetWindowText(csIPAddressByte.GetString());
	csIPAddressByte.Format("%d",m_TimeService_SockAddr.sin_addr.S_un.S_un_b.s_b3);
	m_pNTP_Address_3->SetWindowText(csIPAddressByte.GetString());
	csIPAddressByte.Format("%d",m_TimeService_SockAddr.sin_addr.S_un.S_un_b.s_b4);
	m_pNTP_Address_4->SetWindowText(csIPAddressByte.GetString());

	m_TimeService_SockAddr.sin_family = AF_INET;		// Set the address type 
	
	// Don't use htons when using the network library (it is applied automatically in the network library) 
	m_TimeService_SockAddr.sin_port = htons(NTP_Port);   // Set the NTP port (Decimal 123) Do use htons if using raw sockets

	CString csPort;
	csPort.Format("%d",NTP_Port);
	m_pNTP_PORT->SetWindowText(csPort.GetString());
}

void CCaseStudy_TimeServiceClient_GUIDlg::Populate_NTP_Server_List()
{
	m_pNTP_URL_List->InsertString(0,"nist1-ny.ustiming.org");
	m_pNTP_Location_List->InsertString(0,"New York City, NY");

	m_pNTP_URL_List->InsertString(0,"nist1-nj.ustiming.org");
	m_pNTP_Location_List->InsertString(0,"Bridgewater, NJ");

	m_pNTP_URL_List->InsertString(0,"nist1-atl.ustiming.org");
	m_pNTP_Location_List->InsertString(0,"Atlanta, Georgia");

	m_pNTP_URL_List->InsertString(0,"wolfnisttime.com");
	m_pNTP_Location_List->InsertString(0,"Birmingham, Alabama");

	m_pNTP_URL_List->InsertString(0,"nist1-chi.ustiming.org");
	m_pNTP_Location_List->InsertString(0,"Chicago, Illinois");

	m_pNTP_URL_List->InsertString(0,"nist1-lnk.binary.net");
	m_pNTP_Location_List->InsertString(0,"Lincoln, Nebraska");

	m_pNTP_URL_List->InsertString(0,"time-a.timefreq.bldrdoc.gov");
	m_pNTP_Location_List->InsertString(0,"NIST, Boulder, Colorado");

	m_pNTP_URL_List->InsertString(0,"ntp-nist.ldsbc.edu");
	m_pNTP_Location_List->InsertString(0,"LDSBC, Salt Lake City, Utah");
	
	m_pNTP_URL_List->InsertString(0,"nist1-lv.ustiming.org");
	m_pNTP_Location_List->InsertString(0,"Las Vegas, Nevada");  

	m_pNTP_URL_List->InsertString(0,"nist1-la.ustiming.org");
	m_pNTP_Location_List->InsertString(0,"Los Angeles, California");

	m_pNTP_URL_List->InsertString(0,"time.nist.gov");
	m_pNTP_Location_List->InsertString(0,"NIST round robin load equalisation");

	m_pNTP_URL_List->InsertString(0,"time.windows.com");
	m_pNTP_Location_List->InsertString(0,"Windows Time service");
}

void CCaseStudy_TimeServiceClient_GUIDlg::OnLbnSelchangeNtpServerUrlList()
{
	int iSelectionIndex = m_pNTP_URL_List->GetCurSel();
	CString cStrURL;
	m_pNTP_Location_List->SetCurSel(iSelectionIndex);
	m_pNTP_URL_List->GetText(iSelectionIndex,cStrURL);
	m_pSelectedURL->SetWindowTextA(cStrURL.GetString());
	SetUp_TimeService_AddressStruct(cStrURL);
}

void CCaseStudy_TimeServiceClient_GUIDlg::UpdateStatisticsDisplay()
{
	CString csStr;
	csStr.Format("%d",m_iNumRequestsSent);
	m_pNumNTPRequestsSent->SetWindowTextA(csStr.GetString());
	csStr.Format("%d",m_iNumResponsesReceived);
	m_pNumNTPResponsesReceived->SetWindowTextA(csStr.GetString());
}


void CCaseStudy_TimeServiceClient_GUIDlg::OnBnClickedCancel()
{
	closesocket(m_TimeService_Socket);	// If the socket was open, Close the socket
	CDialog::OnCancel();
}
